/*
*  DISCLAIMER:
*
*      THIS SOFTWARE, SOURCE CODE AND ASSOCIATED MATERIALS INCLUDING BUT NOT LIMITED TO TUTORIALS,
*      GUIDES AND COMMENTARY PROVIDED WITH THIS EXERCISE ARE ONLY DESIGNED FOR REFERENCE PURPOSES
*      TO GIVE AN EXAMPLE TO LICENSEE FOR THEIR OWN NECESSARY DEVELOPMENT OF THEIR OWN SOFTWARE AND/OR
*      APPLICATION. IT IS NOT DESIGNED FOR ANY SPECIAL PURPOSE, SERIAL PRODUCTION OR USE IN MEDICAL,
*      MILITARY, AIR CRAFT, AVIATION, SPACE OF LIFE SUPPORT EQUIPMENT.
*
*      TO THE EXTENT PERMITTED BY LAW, THE EXERCISE SOFTWARE AND/OR SOURCE CODE AND/OR AND ASSOCIATED
*      MATERIALS IS PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND AND ONLY FOR REFERENCE PURPOSES.
*
*      SYNAPTIC LABORATORIES LTD. MAKES NO WARRANTIES, EITHER EXPRESS OR IMPLIED, WITH RESPECT TO THE
*      LICENSED SOFTWARE AND/OR SOURCE CODE AND/OR ASSOCIATED MATERIALS, CONFIDENTIAL INFORMATION AND
*      DOCUMENTATION PROVIDED HEREUNDER.
*
*      SYNAPTIC LABORATORIES LTD. SPECIFICALLY DISCLAIMS THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
*      FITNESS FOR A PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT OF ANY INTELLECTUAL
*      PROPERTY RIGHT OF ANY THIRD PARTY WITH REGARD TO THE SOFTWARE, DOCUMENTATION (SCHEMATICS ETC.),
*      SOURCE CODE AND ASSOCIATED MATERIALS, CONFIDENTIAL INFORMATION AND DOCUMENTATION.
*
*      ANY USE, COMPILATION AND TESTING OF THE SOFTWARE AND/OR SOURCE CODE IS AT LICENSEE`S OWN RISK
*      AND LICENSEE IS OBLIGED TO CONDUCT EXTENSIVE TESTS TO AVOID ANY ERRORS AND FAILURE IN THE
*      COMPILED SOURCE CODE, DOCUMENTATION (SCHEMATICS ETC.) AND THE HEREFROM GENERATED SOFTWARE
*      OF LICENSEE.
*
*      EXCEPT FOR WILFULL INTENT SYNAPTIC LABORATORIES LTD. SHALL IN NO EVENT BE ENTITLED TO OR LIABLE
*      FOR ANY INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY KIND OR NATURE, INCLUDING,
*      WITHOUT LIMITATION, BUSINESS INTERRUPTION COSTS, LOSS OF PROFIT OR REVENUE, LOSS OF DATA,
*      PROMOTIONAL OR MANUFACTURING EXPENSES, OVERHEAD, COSTS OR EXPENSES ASSOCIATED WITH WARRANTY
*      OR INTELLECTUAL PROPERTY INFRINGEMENT CLAIMS, INJURY TO REPUTATION OR LOSS OF CUSTOMERS.
*
*/

#include "sll_ca_hbc_t001_reg.h"
#include "FreeRTOS/portable/Hal.h"

#define FlashWriteUserCmd(b,o,d)    sll_hyperflash_write_cmd(b, o, d)
#define FlashReadUserCmd(b,o)       sll_hyperflash_read_cmd(b, o)

/******************************************************************************
*
* sll_psram_reg_read  - Read 16-bit value to PSRAM(@CS1) Register over the control port
*                        with CA format {cad_high[15:0], cad_low[31:0]}
*
* RETURNS: 16-bit value
*
* ERRNO  : None
*/
uint16_t sll_hyperram_reg_read  (uint32_t base, uint32_t cad_high, uint32_t cad_low) {

  uint32_t cad_rdata_valid = 0;
  uint16_t cad_rdata;
  volatile uint32_t reg_status  = 0;
  
  //send command/address/data field
  IOWR_SLL_MBMC_REG_CAD_LO(base, cad_low);
  IOWR_SLL_MBMC_REG_CAD_HI(base, cad_high);
  IOWR_SLL_MBMC_REG_CMD(base, SLL_MBMC_RAM_CS1_RD);
   
  //wait until read data is available is ready
  while (cad_rdata_valid == 0) {
    reg_status      = IORD_SLL_MBMC_REG_STATUS(base);
    cad_rdata_valid = (reg_status & SLL_MBMC_REG_STATUS_RDATA_VLD) != 0;
  }
  
  cad_rdata = IORD_SLL_MBMC_REG_RDATA(base);
  
  return (uint16_t) cad_rdata;
  
};

/******************************************************************************
*
* sll_psram_reg_write  - Write 32-bit value to PSRAM(@CS1) Register over the control port
*                        with CA format {cad_high[15:0], cad_low[31:0]}
*
* RETURNS: None
*
* ERRNO  : None
*/
void    sll_hyperram_reg_write (uint32_t base, uint32_t cad_high, uint32_t cad_low,  uint16_t wdata) {

  uint32_t cad_cmd_rdy = 0;
  volatile uint32_t reg_status  = 0;
  
  //send command/address/data field
  IOWR_SLL_MBMC_REG_CAD_LO  (base, cad_low);
  IOWR_SLL_MBMC_REG_CAD_HI  (base, cad_high);
  IOWR_SLL_MBMC_REG_CAD_DATA(base, wdata);
  IOWR_SLL_MBMC_REG_CMD     (base, SLL_MBMC_RAM_CS1_WR);
  
  //wait until command is ready
  while (cad_cmd_rdy == 0) {
    reg_status    = IORD_SLL_MBMC_REG_STATUS(base);
    if ((reg_status & SLL_MBMC_REG_STATUS_CAD_ACTIVE) == 0) {
      cad_cmd_rdy = 1;
    }   
  }
}

/******************************************************************************
*
* sll_flash_write_cmd  - Write 16-bit value to Flash(@CS0) over the control port
*                        with address offset
*
* RETURNS: None
*
* ERRNO  : None
*/
void    sll_hyperflash_write_cmd (uint32_t base, uint32_t offset, uint16_t wdata) {

  uint32_t cad_cmd_rdy = 0;
  volatile uint32_t reg_status  = 0;
  
  //send command/address/data field
  IOWR_SLL_MBMC_REG_CAD_HI  (base, 0x0000);
  IOWR_SLL_MBMC_REG_CAD_LO  (base, offset);
  IOWR_SLL_MBMC_REG_CAD_DATA(base, wdata);
  IOWR_SLL_MBMC_REG_CMD     (base, SLL_MBMC_FLASH_CS0_WR);
  
  //wait until command is ready
  while (cad_cmd_rdy == 0) {
     reg_status   = IORD_SLL_MBMC_REG_STATUS(base);
     if ((reg_status & SLL_MBMC_REG_STATUS_CAD_ACTIVE) == 0) {
      cad_cmd_rdy = 1;
    }   
  }
}

/******************************************************************************
*
* sll_flash_read_cmd  - Read 16-bit value from Flash(@CS0) over the control port
*                        with address offset
*
* RETURNS: 16-bit value
*
* ERRNO  : None
*/
uint16_t sll_hyperflash_read_cmd  (uint32_t base, uint32_t offset) {

  uint32_t cad_rdata_valid = 0;
  uint16_t cad_rdata;
  volatile uint32_t reg_status  = 0;
  
  //send command/address/data field
  IOWR_SLL_MBMC_REG_CAD_HI(base, 0x0000);
  IOWR_SLL_MBMC_REG_CAD_LO(base, offset);
  IOWR_SLL_MBMC_REG_CMD(base, SLL_MBMC_FLASH_CS0_RD);
   
  //wait until read data is available is ready
  while (cad_rdata_valid == 0) {
     reg_status      = IORD_SLL_MBMC_REG_STATUS(base);
     cad_rdata_valid = (reg_status & SLL_MBMC_REG_STATUS_RDATA_VLD) != 0;
  }
  
  cad_rdata = IORD_SLL_MBMC_REG_RDATA(base);
  
  return (uint16_t) cad_rdata;
  
};  


/******************************************************************************
*
* DelayMicroseconds - Performs a delay.
*
* RETURNS: void
*
*/

void SLL_DelayMicroseconds(int microseconds) {
	Hal_Delay(microseconds * CLK_FREQ / 10000000);
}
