/*
*  DISCLAIMER:
*
*      THIS SOFTWARE, SOURCE CODE AND ASSOCIATED MATERIALS INCLUDING BUT NOT LIMITED TO TUTORIALS,
*      GUIDES AND COMMENTARY PROVIDED WITH THIS EXERCISE ARE ONLY DESIGNED FOR REFERENCE PURPOSES
*      TO GIVE AN EXAMPLE TO LICENSEE FOR THEIR OWN NECESSARY DEVELOPMENT OF THEIR OWN SOFTWARE AND/OR
*      APPLICATION. IT IS NOT DESIGNED FOR ANY SPECIAL PURPOSE, SERIAL PRODUCTION OR USE IN MEDICAL,
*      MILITARY, AIR CRAFT, AVIATION, SPACE OF LIFE SUPPORT EQUIPMENT.
*
*      TO THE EXTENT PERMITTED BY LAW, THE EXERCISE SOFTWARE AND/OR SOURCE CODE AND/OR AND ASSOCIATED
*      MATERIALS IS PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND AND ONLY FOR REFERENCE PURPOSES.
*
*      SYNAPTIC LABORATORIES LTD. MAKES NO WARRANTIES, EITHER EXPRESS OR IMPLIED, WITH RESPECT TO THE
*      LICENSED SOFTWARE AND/OR SOURCE CODE AND/OR ASSOCIATED MATERIALS, CONFIDENTIAL INFORMATION AND
*      DOCUMENTATION PROVIDED HEREUNDER.�
*
*      SYNAPTIC LABORATORIES LTD. SPECIFICALLY DISCLAIMS THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
*      FITNESS FOR A PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT OF ANY INTELLECTUAL
*      PROPERTY RIGHT OF ANY THIRD PARTY WITH REGARD TO THE SOFTWARE, DOCUMENTATION (SCHEMATICS ETC.),
*      SOURCE CODE AND ASSOCIATED MATERIALS, CONFIDENTIAL INFORMATION AND DOCUMENTATION.
*
*      ANY USE, COMPILATION AND TESTING OF THE SOFTWARE AND/OR SOURCE CODE IS AT LICENSEE`S OWN RISK
*      AND LICENSEE IS OBLIGED TO CONDUCT EXTENSIVE TESTS TO AVOID ANY ERRORS AND FAILURE IN THE
*      COMPILED SOURCE CODE, DOCUMENTATION (SCHEMATICS ETC.) AND THE HEREFROM GENERATED SOFTWARE
*      OF LICENSEE.
*
*      EXCEPT FOR WILFULL INTENT SYNAPTIC LABORATORIES LTD. SHALL IN NO EVENT BE ENTITLED TO OR LIABLE
*      FOR ANY INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY KIND OR NATURE, INCLUDING,
*      WITHOUT LIMITATION, BUSINESS INTERRUPTION COSTS, LOSS OF PROFIT OR REVENUE, LOSS OF DATA,
*      PROMOTIONAL OR MANUFACTURING EXPENSES, OVERHEAD, COSTS OR EXPENSES ASSOCIATED WITH WARRANTY
*      OR INTELLECTUAL PROPERTY INFRINGEMENT CLAIMS, INJURY TO REPUTATION OR LOSS OF CUSTOMERS.
*
*/


#ifndef __INC_H_SLL_HYPERFLASH_DRVR
#define __INC_H_SLL_HYPERFLASH_DRVR

#include "sll_ca_hbc_t001_reg.h"


#ifdef __cplusplus
 extern "C" {
#endif /* __cplusplus */


#define SLL_HYPERFLASH_DRVR_VERSION   "21.3.2"   //  Year.Quarter.Minor
#define HYPERFLASH_SECTORSIZE (256*1024) // 256 kB
#define SA_OFFSET_MASK	0xFFFE0000   /* mask off the offset */

/* LLD Command Definition */
#define NOR_CFI_QUERY_CMD                (0x98)
#define NOR_ERASE_SETUP_CMD              (0x80)
#define NOR_RESET_CMD                    (0xF0)
#define NOR_SECTOR_ERASE_CMD             (0x30)
#define NOR_WRITE_BUFFER_LOAD_CMD        (0x25)
#define NOR_WRITE_BUFFER_PGM_CONFIRM_CMD (0x29) 

#define NOR_STATUS_REG_READ_CMD          (0x70)
#define NOR_STATUS_REG_CLEAR_CMD         (0x71)

/* Command code definition */
#define NOR_AUTOSELECT_CMD               (0x90)
#define NOR_PROGRAM_CMD                  (0xA0)
#define NOR_UNLOCK_DATA1                 (0xAA)
#define NOR_UNLOCK_DATA2                 (0x55)



/* polling return status */
typedef enum {
	DEV_STATUS_UNKNOWN = 0,
	DEV_NOT_BUSY,
	DEV_BUSY,
	DEV_EXCEEDED_TIME_LIMITS,
	DEV_SUSPEND,
	DEV_WRITE_BUFFER_ABORT,
	DEV_BYTES_PER_OP_WRONG,
	DEV_ERASE_ERROR,       
	DEV_PROGRAM_ERROR,       
	DEV_SECTOR_LOCK,
	DEV_PROGRAM_SUSPEND,       /* Device is in program suspend mode */
	DEV_PROGRAM_SUSPEND_ERROR, /* Device program suspend error */
	DEV_ERASE_SUSPEND,         /* Device is in erase suspend mode */
	DEV_ERASE_SUSPEND_ERROR    /* Device erase suspend error */
} DEVSTATUS;

#define DEV_RDY_MASK          (0x80) /* Device Ready Bit */
#define DEV_ERASE_SUSP_MASK   (0x40) /* Erase Suspend Bit */
#define DEV_ERASE_MASK        (0x20) /* Erase Status Bit */
#define DEV_PROGRAM_MASK      (0x10) /* Program Status Bit */
#define DEV_RFU_MASK          (0x08) /* Reserved */
#define DEV_PROGRAM_SUSP_MASK (0x04) /* Program Suspend Bit */
#define DEV_SEC_LOCK_MASK     (0x02) /* Sector lock Bit */
#define DEV_BANK_MASK         (0x01) /* Operation in current bank */

/*****************************************************
* Define Flash read/write macro to be used by LLD    *
*****************************************************/
#define FLASH_OFFSET(b,o)     (*(( (volatile uint16_t*)(b) ) + (o)))


#define LLD_DEV_READ_MASK     0x0000FFFF
#define LLD_UNLOCK_ADDR1      0x00000555
#define LLD_UNLOCK_ADDR2      0x000002AA
#define LLD_BYTES_PER_OP      0x00000002
#define LLD_CFI_UNLOCK_ADDR1  0x00000055


/* public function prototypes */


extern DEVSTATUS sll_nor_flash_drvr_WriteBufferProgramOp
(
uint32_t base_csr,       /* Control port base address */
uint32_t offset,         /* address offset from base address */
uint32_t word_cnt,       /* number of words to program */
uint16_t *data_buf       /* buffer containing data to program */
);


extern DEVSTATUS lld_ProgramOp
(
uint32_t base_csr,       /* Control port base address */
uint32_t offset,         /* address offset from base address */
uint16_t write_data      /* variable containing data to program */
);


extern DEVSTATUS sll_nor_flash_drvr_SectorEraseOp
(
uint32_t base_csr,       /* Control port base address */
uint32_t offset          /* address offset from base address */
);



uint16_t sll_nor_flash_drvr_ReadCfiWord
(
uint32_t base_csr,       /* Control port base address */
uint32_t offset          /* address offset from base address */
);



DEVSTATUS sll_nor_flash_drvr_memcpy
(
uint32_t base_csr,       /* Control port base address */
uint32_t offset,         /* address offset from base address */
uint32_t words_cnt,      /* number of words to program */
uint16_t *data_buf       /* buffer containing data to program */
);


extern uint16_t sll_nor_flash_drvr_Poll
(
uint32_t base_csr,       /* Control port base address */
uint32_t offset          /* address offset from base address */
);

extern unsigned int sll_nor_flash_drvr_GetDeviceId
(
uint32_t base_csr,       /* Control port base address */
uint32_t offset
);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __INC_H_SLL_HYPERFLASH_DRVR  */
